@@ -41,31 +41,31 @@ class AutoExtendableTC(TestCase):
 
 
     def test_autoextend(self):
-        self.failIf('generate_server' in dir(ValueGenerator))
+        self.assertFalse('generate_server' in dir(ValueGenerator))
         class MyValueGenerator(ValueGenerator):
             def generate_server(self, index):
                 return attrname
-        self.failUnless('generate_server' in dir(ValueGenerator))
+        self.assertTrue('generate_server' in dir(ValueGenerator))
 
 
     def test_bad_signature_detection(self):
-        self.failIf('generate_server' in dir(ValueGenerator))
+        self.assertFalse('generate_server' in dir(ValueGenerator))
         try:
             class MyValueGenerator(ValueGenerator):
                 def generate_server(self):
                     pass
         except TypeError:
-            self.failIf('generate_server' in dir(ValueGenerator))
+            self.assertFalse('generate_server' in dir(ValueGenerator))
         else:
             self.fail('TypeError not raised')
 
 
     def test_signature_extension(self):
-        self.failIf('generate_server' in dir(ValueGenerator))
+        self.assertFalse('generate_server' in dir(ValueGenerator))
         class MyValueGenerator(ValueGenerator):
             def generate_server(self, index, foo):
                 pass
-        self.failUnless('generate_server' in dir(ValueGenerator))
+        self.assertTrue('generate_server' in dir(ValueGenerator))
 
 
 if __name__ == '__main__':
