@@ -28,7 +28,7 @@ from yams.schema import role_name
 from cubicweb import ValidationError
 from cubicweb.schema import (META_RTYPES, WORKFLOW_RTYPES,
                              RQLConstraint, RQLUniqueConstraint)
-from cubicweb.selectors import is_instance
+from cubicweb.predicates import is_instance
 from cubicweb.uilib import soup2xhtml
 from cubicweb.server import hook
 
@@ -301,10 +301,11 @@ class _DelayedDeleteOp(hook.DataOperationMixIn, hook.Operation):
     def precommit_event(self):
         session = self.session
         pendingeids = session.transaction_data.get('pendingeids', ())
+        neweids = session.transaction_data.get('neweids', ())
         eids_by_etype_rtype = {}
         for eid, rtype in self.get_data():
-            # don't do anything if the entity is being deleted
-            if eid not in pendingeids:
+            # don't do anything if the entity is being created or deleted
+            if not (eid in pendingeids or eid in neweids):
                 etype = session.describe(eid)[0]
                 key = (etype, rtype)
                 if key not in eids_by_etype_rtype:
