@@ -1,4 +1,4 @@
-# copyright 2003-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# copyright 2003-2011 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 # contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
 #
 # This file is part of CubicWeb.
@@ -25,10 +25,9 @@ class PyViewsTC(CubicWebTC):
                                          pyvalue=[[1, 'a'], [2, 'b']])
         content = view.render(pyvalue=[[1, 'a'], [2, 'b']],
                               headers=['num', 'char'])
-        self.assertEqual(content.strip(), '''<table class="listing">
-<thead><tr><th>num</th><th>char</th></tr>
-</thead><tbody><tr><td>1</td><td>a</td></tr>
-<tr><td>2</td><td>b</td></tr>
+        self.assertEqual(content.strip(), '''<table class="listing"><tbody>\
+<tr class="even" onmouseout="$(this).removeClass(&quot;highlighted&quot;)" onmouseover="$(this).addClass(&quot;highlighted&quot;);"><td >1</td><td >a</td></tr>
+<tr class="odd" onmouseout="$(this).removeClass(&quot;highlighted&quot;)" onmouseover="$(this).addClass(&quot;highlighted&quot;);"><td >2</td><td >b</td></tr>
 </tbody></table>''')
 
     def test_pyvallist(self):
